/* Machine-specific pthread type layouts.  C-SKY version.
   Copyright (C) 2018-2019 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library.  If not, see
   <http://www.gnu.org/licenses/>.  */

#ifndef _BITS_PTHREADTYPES_ARCH_H
#define _BITS_PTHREADTYPES_ARCH_H	1

#include <endian.h>

#define __SIZEOF_PTHREAD_ATTR_T			36
#define __SIZEOF_PTHREAD_MUTEX_T		24
#define __SIZEOF_PTHREAD_MUTEXATTR_T		4
#define __SIZEOF_PTHREAD_COND_T			48
#define __SIZEOF_PTHREAD_CONDATTR_T		4
#define __SIZEOF_PTHREAD_RWLOCK_T		32
#define __SIZEOF_PTHREAD_RWLOCKATTR_T		8
#define __SIZEOF_PTHREAD_BARRIER_T		20
#define __SIZEOF_PTHREAD_BARRIERATTR_T		4

/* Data structure for mutex handling.  */
#define __PTHREAD_COMPAT_PADDING_MID
#define __PTHREAD_COMPAT_PADDING_END
#define __PTHREAD_MUTEX_LOCK_ELISION		0
#define __PTHREAD_MUTEX_NUSERS_AFTER_KIND	1
#define __PTHREAD_MUTEX_USE_UNION		1

#define __LOCK_ALIGNMENT
#define __ONCE_ALIGNMENT

/* Paddings in this structure are not strictly necessary on C-SKY.
   They are left for extensibility as most other architecture do so.  */
struct __pthread_rwlock_arch_t
{
  unsigned int __readers;
  unsigned int __writers;
  unsigned int __wrphase_futex;
  unsigned int __writers_futex;
  unsigned int __pad3;
  unsigned int __pad4;
#if __BYTE_ORDER == __BIG_ENDIAN
  unsigned char __pad1;
  unsigned char __pad2;
  unsigned char __shared;
  unsigned char __flags;
#else
  unsigned char __flags;
  unsigned char __shared;
  unsigned char __pad1;
  unsigned char __pad2;
#endif
  int __cur_writer;
};

#define __PTHREAD_RWLOCK_ELISION_EXTRA 0

#endif
