#include <bits/wordsize.h>

#if __WORDSIZE == 64
# define __PTHREAD_MUTEX_NUSERS_OFFSET   12
# define __PTHREAD_MUTEX_KIND_OFFSET     16
# define __PTHREAD_MUTEX_SPINS_OFFSET    20
# define __PTHREAD_MUTEX_ELISION_OFFSET  22
# define __PTHREAD_MUTEX_LIST_OFFSET     24
#else
# define __PTHREAD_MUTEX_NUSERS_OFFSET   16
# define __PTHREAD_MUTEX_KIND_OFFSET     12
# define __PTHREAD_MUTEX_SPINS_OFFSET    20
# define __PTHREAD_MUTEX_ELISION_OFFSET  22
# define __PTHREAD_MUTEX_LIST_OFFSET     20
#endif
